@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 /*
   NT_auth -  Version 2.0
 
@@ -13,7 +21,7 @@
     Bill Welliver 1999
 
  * Distributed freely under the terms of the GNU General Public License,
- * version 2. See the file COPYING for licensing details
+ * version 2 or later. See the file COPYING for licensing details
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,9 +67,18 @@ extern int debug_enabled;
 extern char Default_NTDomain[DNLEN+1];
 extern const char * errormsg;
 
+/**
+ * Valid_User return codes.
+ *
+ * \retval 0   User authenticated successfully.
+ * \retval 1   Server error.
+ * \retval 2   Group membership error.
+ * \retval 3   Logon error; Incorrect password or username given.
+ */
+int Valid_User(char *UserName, char *Password, char *Group);
+
 /* Debugging stuff */
-#if defined(__GNUC__)			/* this is really a gcc-ism */
-#include <stdio.h>
+#if defined(__GNUC__)           /* this is really a gcc-ism */
 #include <unistd.h>
 static char *__foo;
 #define debug(X...) if (debug_enabled) { \
@@ -84,6 +101,5 @@ debug(char *format,...)
 }
 #endif /* __GNUC__ */
 
-int Valid_User(char *,char *, char *);
-
 #endif
+
