@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -466,13 +466,13 @@ main(int argc, char **argv)
                 if (strchr(buf, '\n') != NULL)
                     break;
             }
-            SEND_BH(HLP_MSG("Input too large"));
+            SEND_ERR("");
             continue;
         }
         user = strtok(buf, " \n");
         if (!user) {
             debug("%s: Invalid request: No Username given\n", argv[0]);
-            SEND_BH(HLP_MSG("Invalid request. No Username"));
+            SEND_ERR("Invalid request. No Username");
             continue;
         }
         rfc1738_unescape(user);
@@ -495,12 +495,11 @@ main(int argc, char **argv)
             extension_dn = strtok(NULL, " \n");
             if (!extension_dn) {
                 debug("%s: Invalid request: Extension DN configured, but none sent.\n", argv[0]);
-                SEND_BH(HLP_MSG("Invalid Request. Extension DN required"));
+                SEND_ERR("Invalid Request. Extension DN required.");
                 continue;
             }
             rfc1738_unescape(extension_dn);
         }
-        const char *broken = nullptr;
         while (!found && user && (group = strtok(NULL, " \n")) != NULL) {
             rfc1738_unescape(group);
 
@@ -510,7 +509,6 @@ recover:
                 if (strstr(ldapServer, "://") != NULL) {
                     rc = ldap_initialize(&ld, ldapServer);
                     if (rc != LDAP_SUCCESS) {
-                        broken = HLP_MSG("Unable to connect to LDAP server");
                         fprintf(stderr, "%s: ERROR: Unable to connect to LDAPURI:%s\n", argv[0], ldapServer);
                         break;
                     }
@@ -532,8 +530,7 @@ recover:
                     } else
 #endif
                         if ((ld = ldap_init(ldapServer, port)) == NULL) {
-                            broken = HLP_MSG("Unable to connect to LDAP server");
-                            fprintf(stderr, "ERROR: %s:%s port:%d\n", broken, ldapServer, port);
+                            fprintf(stderr, "ERROR: Unable to connect to LDAP server:%s port:%d\n", ldapServer, port);
                             break;
                         }
                 if (connect_timeout)
@@ -544,8 +541,8 @@ recover:
                     version = LDAP_VERSION3;
                 }
                 if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) != LDAP_SUCCESS) {
-                    broken = HLP_MSG("Could not set LDAP_OPT_PROTOCOL_VERSION");
-                    fprintf(stderr, "ERROR: %s %d\n", broken, version);
+                    fprintf(stderr, "ERROR: Could not set LDAP_OPT_PROTOCOL_VERSION %d\n",
+                            version);
                     ldap_unbind(ld);
                     ld = NULL;
                     break;
@@ -556,8 +553,7 @@ recover:
                         fprintf(stderr, "FATAL: TLS requires LDAP version 3\n");
                         exit(1);
                     } else if (ldap_start_tls_s(ld, NULL, NULL) != LDAP_SUCCESS) {
-                        broken = HLP_MSG("Could not Activate TLS connection");
-                        fprintf(stderr, "ERROR: %s\n", broken);
+                        fprintf(stderr, "ERROR: Could not Activate TLS connection\n");
                         ldap_unbind(ld);
                         ld = NULL;
                         break;
@@ -574,8 +570,7 @@ recover:
                 if (binddn && bindpasswd && *binddn && *bindpasswd) {
                     rc = ldap_simple_bind_s(ld, binddn, bindpasswd);
                     if (rc != LDAP_SUCCESS) {
-                        broken = HLP_MSG("could not bind");
-                        fprintf(stderr, PROGRAM_NAME ": WARNING: %s to binddn '%s'\n", broken, ldap_err2string(rc));
+                        fprintf(stderr, PROGRAM_NAME ": WARNING: could not bind to binddn '%s'\n", ldap_err2string(rc));
                         ldap_unbind(ld);
                         ld = NULL;
                         break;
@@ -583,24 +578,20 @@ recover:
                 }
                 debug("Connected OK\n");
             }
-            int searchResult = searchLDAP(ld, group, user, extension_dn);
-            if (searchResult == 0) {
+            if (searchLDAP(ld, group, user, extension_dn) == 0) {
                 found = 1;
                 break;
-            } else if (searchResult < 0) {
+            } else {
                 if (tryagain) {
                     tryagain = 0;
                     ldap_unbind(ld);
                     ld = NULL;
                     goto recover;
                 }
-                broken = HLP_MSG("LDAP search error");
             }
         }
         if (found)
             SEND_OK("");
-        else if (broken)
-            SEND_BH(broken);
         else {
             SEND_ERR("");
         }
@@ -722,7 +713,7 @@ searchLDAPGroup(LDAP * ld, char *group, char *member, char *extension_dn)
 
     if (build_filter(filter, sizeof(filter), searchfilter, member, group) != 0) {
         fprintf(stderr, PROGRAM_NAME ": ERROR: Failed to construct LDAP search filter. filter=\"%s\", user=\"%s\", group=\"%s\"\n", filter, member, group);
-        return -1;
+        return 1;
     }
     debug("group filter '%s', searchbase '%s'\n", filter, searchbase);
 
@@ -741,7 +732,7 @@ searchLDAPGroup(LDAP * ld, char *group, char *member, char *extension_dn)
             }
 #endif
             ldap_msgfree(res);
-            return -1;
+            return 1;
         }
     }
     entry = ldap_first_entry(ld, res);
@@ -788,7 +779,7 @@ searchLDAP(LDAP * ld, char *group, char *login, char *extension_dn)
                 }
 #endif
                 ldap_msgfree(res);
-                return -1;
+                return 1;
             }
         }
         entry = ldap_first_entry(ld, res);
