@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -10,7 +10,6 @@
 
 #include "squid.h"
 #include "md5.h"
-#include "Mem.h"
 #include "StatCounters.h"
 #include "Store.h"
 #include "store_key_md5.h"
@@ -18,6 +17,7 @@
 #if USE_CACHE_DIGESTS
 
 #include "CacheDigest.h"
+#include "util.h"
 
 /* local types */
 
@@ -35,12 +35,12 @@ static void cacheDigestHashKey(const CacheDigest * cd, const cache_key * key);
 static uint32_t hashed_keys[4];
 
 static void
-cacheDigestInit(CacheDigest * cd, uint64_t capacity, uint8_t bpe)
+cacheDigestInit(CacheDigest * cd, int capacity, int bpe)
 {
-    const uint32_t mask_size = cacheDigestCalcMaskSize(capacity, bpe);
+    const size_t mask_size = cacheDigestCalcMaskSize(capacity, bpe);
     assert(cd);
     assert(capacity > 0 && bpe > 0);
-    assert(mask_size != 0);
+    assert(mask_size > 0);
     cd->capacity = capacity;
     cd->bits_per_entry = bpe;
     cd->mask_size = mask_size;
@@ -50,7 +50,7 @@ cacheDigestInit(CacheDigest * cd, uint64_t capacity, uint8_t bpe)
 }
 
 CacheDigest *
-cacheDigestCreate(uint64_t capacity, uint8_t bpe)
+cacheDigestCreate(int capacity, int bpe)
 {
     CacheDigest *cd = (CacheDigest *)memAllocate(MEM_CACHE_DIGEST);
     assert(SQUID_MD5_DIGEST_LENGTH == 16);  /* our hash functions rely on 16 byte keys */
@@ -97,7 +97,7 @@ cacheDigestClear(CacheDigest * cd)
 
 /* changes mask size, resets bits to 0, preserves "cd" pointer */
 void
-cacheDigestChangeCap(CacheDigest * cd, uint64_t new_cap)
+cacheDigestChangeCap(CacheDigest * cd, int new_cap)
 {
     assert(cd);
     cacheDigestClean(cd);
@@ -278,12 +278,12 @@ cacheDigestReport(CacheDigest * cd, const char *label, StoreEntry * e)
     storeAppendPrintf(e, "%s digest: size: %d bytes\n",
                       label ? label : "", stats.bit_count / 8
                      );
-    storeAppendPrintf(e, "\t entries: count: %" PRIu64 " capacity: %" PRIu64 " util: %d%%\n",
+    storeAppendPrintf(e, "\t entries: count: %d capacity: %d util: %d%%\n",
                       cd->count,
                       cd->capacity,
                       xpercentInt(cd->count, cd->capacity)
                      );
-    storeAppendPrintf(e, "\t deletion attempts: %" PRIu64 "\n",
+    storeAppendPrintf(e, "\t deletion attempts: %d\n",
                       cd->del_count
                      );
     storeAppendPrintf(e, "\t bits: per entry: %d on: %d capacity: %d util: %d%%\n",
@@ -297,18 +297,16 @@ cacheDigestReport(CacheDigest * cd, const char *label, StoreEntry * e)
                      );
 }
 
-uint32_t
-cacheDigestCalcMaskSize(uint64_t cap, uint8_t bpe)
+size_t
+cacheDigestCalcMaskSize(int cap, int bpe)
 {
-    uint64_t bitCount = (cap * bpe) + 7;
-    assert(bitCount < INT_MAX); // dont 31-bit overflow later
-    return static_cast<uint32_t>(bitCount / 8);
+    return (size_t) (cap * bpe + 7) / 8;
 }
 
 static void
 cacheDigestHashKey(const CacheDigest * cd, const cache_key * key)
 {
-    const uint32_t bit_count = cd->mask_size * 8;
+    const unsigned int bit_count = cd->mask_size * 8;
     unsigned int tmp_keys[4];
     /* we must memcpy to ensure alignment */
     memcpy(tmp_keys, key, sizeof(tmp_keys));
