@@ -1,24 +1,31 @@
+/*
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #ifndef __ASYNC_IO_H__
 #define __ASYNC_IO_H__
 
-#if USE_DISKIO_AIO
+#if HAVE_DISKIO_MODULE_AIO
 
 #if _SQUID_WINDOWS_
-#include "aio_win32.h"
+#include "DiskIO/AIO/aio_win32.h"
 #else
 #if HAVE_AIO_H
 #include <aio.h>
 #endif
 #endif
 
-/* for FREE* */
-#include "typedefs.h"
+#include "mem/forward.h"
 
-#define MAX_ASYNCOP		128
+#define MAX_ASYNCOP     128
 
 typedef enum {
-    AQ_STATE_NONE,		/* Not active/uninitialised */
-    AQ_STATE_SETUP		/* Initialised */
+    AQ_STATE_NONE,      /* Not active/uninitialised */
+    AQ_STATE_SETUP      /* Initialised */
 } async_queue_state_t;
 
 typedef enum {
@@ -61,9 +68,10 @@ struct _async_queue_entry {
 
 struct _async_queue {
     async_queue_state_t aq_state;
-    async_queue_entry_t aq_queue[MAX_ASYNCOP];	/* queued ops */
-    int aq_numpending;		/* Num of pending ops */
+    async_queue_entry_t aq_queue[MAX_ASYNCOP];  /* queued ops */
+    int aq_numpending;      /* Num of pending ops */
 };
 
-#endif /* USE_DISKIO_AIO */
+#endif /* HAVE_DISKIO_MODULE_AIO */
 #endif /* __ASYNC_IO_H_ */
+
