@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -18,6 +18,7 @@
 #include "icp_opcode.h"
 #include "ip/Address.h"
 #include "LogTags.h"
+#include "store_key_md5.h"
 #include "StoreClient.h"
 
 class HttpRequest;
@@ -44,7 +45,7 @@ struct _icp_common_t {
     uint32_t shostid;
 
 /// \todo I don't believe this header is included in non-c++ code anywhere
-///		the struct should become a public POD class and kill these ifdef.
+///     the struct should become a public POD class and kill these ifdef.
 #ifdef __cplusplus
 
     _icp_common_t();
@@ -59,14 +60,6 @@ typedef struct _icp_common_t icp_common_t;
 
 #ifdef __cplusplus
 
-/// \ingroup ServerProtocolICPAPI
-inline icp_opcode & operator++ (icp_opcode & aCode)
-{
-    int tmp = (int) aCode;
-    aCode = (icp_opcode) (++tmp);
-    return aCode;
-}
-
 /**
  \ingroup ServerProtocolICPAPI
  \todo mempool this
@@ -123,7 +116,7 @@ void icpCreateAndSend(icp_opcode, int flags, char const *url, int reqnum, int pa
 icp_opcode icpGetCommonOpcode();
 
 /// \ingroup ServerProtocolICPAPI
-int icpUdpSend(int, const Ip::Address &, icp_common_t *, LogTags, int);
+int icpUdpSend(int, const Ip::Address &, icp_common_t *, const LogTags &, int);
 
 /// \ingroup ServerProtocolICPAPI
 LogTags icpLogFromICPCode(icp_opcode opcode);
@@ -159,3 +152,4 @@ int icpSetCacheKey(const cache_key * key);
 const cache_key *icpGetCacheKey(const char *url, int reqnum);
 
 #endif /* SQUID_ICP_H */
+
