@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -59,8 +59,6 @@ public:
 
     virtual StoreSearch *search(String const url, HttpRequest *);
 
-    virtual bool smpAware() const;
-
 private:
     /* migration logic */
     StorePointer store(int const x) const;
@@ -72,6 +70,7 @@ class StoreHashIndexEntry : public StoreEntry
 
 class StoreSearchHashIndex : public StoreSearch
 {
+    CBDATA_CLASS(StoreSearchHashIndex);
 
 public:
     StoreSearchHashIndex(RefCount<StoreHashIndex> sd);
@@ -97,9 +96,6 @@ private:
     bool _done;
     int bucket;
     std::vector<StoreEntry *> entries;
-
-    // keep this last. it plays with private/public
-    CBDATA_CLASS2(StoreSearchHashIndex);
 };
 
 #endif /* SQUID_STOREHASHINDEX_H */
