@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,6 +9,7 @@
 #ifndef SQUID_SWAPDIR_H
 #define SQUID_SWAPDIR_H
 
+#include "mgr/forward.h"
 #include "SquidConfig.h"
 #include "Store.h"
 #include "StoreIOState.h"
@@ -23,6 +24,8 @@ class HttpRequestMethod;
 /* Store dir configuration routines */
 /* SwapDir *sd, char *path ( + char *opt later when the strtok mess is gone) */
 
+typedef int STDIRSELECT(const StoreEntry *);
+
 class ConfigOption;
 
 /// hides memory/disk cache distinction from callers
