@@ -11,16 +11,13 @@
 
 #include "acl/Acl.h"
 #include "acl/Data.h"
-#include "CbDataList.h"
 #include "hier_code.h"
 
-/// \ingroup ACLAPI
 class ACLHierCodeData : public ACLData<hier_code>
 {
-
-public:
     MEMPROXY_CLASS(ACLHierCodeData);
 
+public:
     ACLHierCodeData();
     ACLHierCodeData(ACLHierCodeData const &);
     ACLHierCodeData &operator= (ACLHierCodeData const &);
@@ -31,11 +28,9 @@ public:
     bool empty() const;
     virtual ACLData<hier_code> *clone() const;
 
-    // mask of codes this ACL might match.
+    /// mask of codes this ACL might match.
     bool values[HIER_MAX];
 };
 
-MEMPROXY_CLASS_INLINE(ACLHierCodeData);
-
 #endif /* SQUID_ACLHIERCODEDATA_H */
 
