@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #ifndef _SQUID_AUTH_BASIC_USER_H
 #define _SQUID_AUTH_BASIC_USER_H
 
@@ -16,17 +24,21 @@ namespace Basic
 /** User credentials for the Basic authentication protocol */
 class User : public Auth::User
 {
-public:
     MEMPROXY_CLASS(Auth::Basic::User);
 
-    User(Auth::Config *);
-    ~User();
+public:
+    User(Auth::Config *, const char *requestRealm);
+    virtual ~User();
     bool authenticated() const;
     bool valid() const;
 
     /** Update the cached password for a username. */
     void updateCached(User *from);
-    virtual int32_t ttl() const;
+    virtual int32_t ttl() const override;
+
+    /* Auth::User API */
+    static CbcPointer<Auth::CredentialsCache> Cache();
+    virtual void addToNameCache() override;
 
     char *passwd;
 
@@ -36,9 +48,8 @@ private:
     Auth::UserRequest::Pointer currentRequest;
 };
 
-MEMPROXY_CLASS_INLINE(Auth::Basic::User);
-
 } // namespace Basic
 } // namespace Auth
 
 #endif /* _SQUID_AUTH_BASIC_USER_H */
+
