@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -10,7 +10,6 @@
 #define _SQUID_SRC_AUTH_BASIC_USERREQUEST_H
 
 #include "auth/UserRequest.h"
-#include "MemPool.h"
 
 class ConnStateData;
 class HttpRequest;
@@ -25,14 +24,14 @@ namespace Basic
 
 class UserRequest : public Auth::UserRequest
 {
-public:
     MEMPROXY_CLASS(Auth::Basic::UserRequest);
 
+public:
     UserRequest() {}
     virtual ~UserRequest() { assert(LockCount()==0); }
 
     virtual int authenticated() const;
-    virtual void authenticate(HttpRequest * request, ConnStateData *conn, http_hdr_type type);
+    virtual void authenticate(HttpRequest * request, ConnStateData *conn, Http::HdrType type);
     virtual Auth::Direction module_direction();
     virtual void startHelperLookup(HttpRequest * request, AccessLogEntry::Pointer &al, AUTHCB *, void *);
     virtual const char *credentialsStr();
@@ -44,6 +43,5 @@ private:
 } // namespace Basic
 } // namespace Auth
 
-MEMPROXY_CLASS_INLINE(Auth::Basic::UserRequest);
-
 #endif /* _SQUID_SRC_AUTH_BASIC_USERREQUEST_H */
+
