@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -11,7 +11,8 @@
 #ifndef SQUID_DISK_H_
 #define SQUID_DISK_H_
 
-#include "typedefs.h"
+#include "mem/forward.h"
+#include "typedefs.h" //DRCB, DWCB
 
 class MemBuf;
 
