@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -14,11 +14,12 @@
 #include "esi/Element.h"
 
 class ESIContext;
-/* esiLiteral */
 
-struct esiLiteral : public ESIElement {
+class esiLiteral : public ESIElement
+{
     MEMPROXY_CLASS(esiLiteral);
 
+public:
     esiLiteral(ESISegment::Pointer);
     esiLiteral(ESIContext *, const char *s, int len);
     ~esiLiteral();
@@ -41,7 +42,5 @@ private:
     esiLiteral(esiLiteral const &);
 };
 
-MEMPROXY_CLASS_INLINE(esiLiteral);
-
 #endif /* SQUID_ESILITERAL_H */
 
