@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -19,9 +19,9 @@
 #include "html_quote.h"
 #include "HttpReply.h"
 #include "HttpRequest.h"
-#include "Mem.h"
 #include "MemBuf.h"
 #include "mime.h"
+#include "parser/Tokenizer.h"
 #include "rfc1738.h"
 #include "SquidConfig.h"
 #include "SquidTime.h"
@@ -34,75 +34,76 @@
 #include "MemObject.h"
 #endif
 
+/* gopher type code from rfc. Anawat. */
+#define GOPHER_FILE         '0'
+#define GOPHER_DIRECTORY    '1'
+#define GOPHER_CSO          '2'
+#define GOPHER_ERROR        '3'
+#define GOPHER_MACBINHEX    '4'
+#define GOPHER_DOSBIN       '5'
+#define GOPHER_UUENCODED    '6'
+#define GOPHER_INDEX        '7'
+#define GOPHER_TELNET       '8'
+#define GOPHER_BIN          '9'
+#define GOPHER_REDUNT       '+'
+#define GOPHER_3270         'T'
+#define GOPHER_GIF          'g'
+#define GOPHER_IMAGE        'I'
+
+#define GOPHER_HTML         'h'
+#define GOPHER_INFO         'i'
+
+///  W3 address
+#define GOPHER_WWW          'w'
+#define GOPHER_SOUND        's'
+
+#define GOPHER_PLUS_IMAGE   ':'
+#define GOPHER_PLUS_MOVIE   ';'
+#define GOPHER_PLUS_SOUND   '<'
+
+#define GOPHER_PORT         70
+
+#define TAB                 '\t'
+
+// TODO CODE: should this be a protocol-specific thing?
+#define TEMP_BUF_SIZE       4096
+
+#define MAX_CSO_RESULT      1024
+
 /**
- \defgroup ServerProtocolGopherInternal Server-Side Gopher Internals
- \ingroup ServerProtocolGopherAPI
+ * Gopher Gateway Internals
+ *
  * Gopher is somewhat complex and gross because it must convert from
  * the Gopher protocol to HTTP.
  */
+class GopherStateData
+{
+    CBDATA_CLASS(GopherStateData);
 
-/* gopher type code from rfc. Anawat. */
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_FILE         '0'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_DIRECTORY    '1'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_CSO          '2'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_ERROR        '3'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_MACBINHEX    '4'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_DOSBIN       '5'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_UUENCODED    '6'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_INDEX        '7'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_TELNET       '8'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_BIN          '9'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_REDUNT       '+'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_3270         'T'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_GIF          'g'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_IMAGE        'I'
+public:
+    GopherStateData(FwdState *aFwd) :
+        entry(aFwd->entry),
+        conversion(NORMAL),
+        HTML_header_added(0),
+        HTML_pre(0),
+        type_id(GOPHER_FILE /* '0' */),
+        cso_recno(0),
+        len(0),
+        buf(NULL),
+        fwd(aFwd)
+    {
+        *request = 0;
+        buf = (char *)memAllocate(MEM_4K_BUF);
+        entry->lock("gopherState");
+        *replybuf = 0;
+    }
+    ~GopherStateData() {if(buf) swanSong();}
 
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_HTML         'h' /* HTML */
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_INFO         'i'
-/**
-  \ingroup ServerProtocolGopherInternal
-  W3 address
- */
-#define GOPHER_WWW          'w'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_SOUND        's'
+    /* AsyncJob API emulated */
+    void deleteThis(const char *aReason);
+    void swanSong();
 
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_PLUS_IMAGE   ':'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_PLUS_MOVIE   ';'
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_PLUS_SOUND   '<'
-
-/// \ingroup ServerProtocolGopherInternal
-#define GOPHER_PORT         70
-
-/// \ingroup ServerProtocolGopherInternal
-#define TAB                 '\t'
-/// \ingroup ServerProtocolGopherInternal
-/// \todo CODE: should this be a protocol-specific thing?
-#define TEMP_BUF_SIZE       4096
-/// \ingroup ServerProtocolGopherInternal
-#define MAX_CSO_RESULT      1024
-
-/// \ingroup ServerProtocolGopherInternal
-typedef struct gopher_ds {
+public:
     StoreEntry *entry;
     enum {
         NORMAL,
@@ -120,10 +121,11 @@ typedef struct gopher_ds {
     int len;
     char *buf;          /* pts to a 4k page */
     Comm::ConnectionPointer serverConn;
-    HttpRequest *req;
     FwdState::Pointer fwd;
     char replybuf[BUFSIZ];
-} GopherStateData;
+};
+
+CBDATA_CLASS_INIT(GopherStateData);
 
 static CLCB gopherStateFree;
 static void gopherMimeCreate(GopherStateData *);
@@ -137,13 +139,10 @@ static IOCB gopherReadReply;
 static IOCB gopherSendComplete;
 static PF gopherSendRequest;
 
-/// \ingroup ServerProtocolGopherInternal
 static char def_gopher_bin[] = "www/unknown";
 
-/// \ingroup ServerProtocolGopherInternal
 static char def_gopher_text[] = "text/plain";
 
-/// \ingroup ServerProtocolGopherInternal
 static void
 gopherStateFree(const CommCloseCbParams &params)
 {
@@ -152,21 +151,29 @@ gopherStateFree(const CommCloseCbParams &params)
     if (gopherState == NULL)
         return;
 
-    if (gopherState->entry) {
-        gopherState->entry->unlock("gopherState");
+    gopherState->deleteThis("gopherStateFree");
+}
+
+void
+GopherStateData::deleteThis(const char *)
+{
+    swanSong();
+    delete this;
+}
+
+void
+GopherStateData::swanSong()
+{
+    if (entry)
+        entry->unlock("gopherState");
+
+    if (buf) {
+        memFree(buf, MEM_4K_BUF);
+        buf = nullptr;
     }
-
-    HTTPMSGUNLOCK(gopherState->req);
-
-    gopherState->fwd = NULL;    // refcounted
-
-    memFree(gopherState->buf, MEM_4K_BUF);
-    gopherState->buf = NULL;
-    cbdataFree(gopherState);
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * Create MIME Header for Gopher Data
  */
 static void
@@ -239,47 +246,48 @@ gopherMimeCreate(GopherStateData * gopherState)
     entry->buffer();
     reply->setHeaders(Http::scOkay, "Gatewaying", mime_type, -1, -1, -2);
     if (mime_enc)
-        reply->header.putStr(HDR_CONTENT_ENCODING, mime_enc);
+        reply->header.putStr(Http::HdrType::CONTENT_ENCODING, mime_enc);
 
     entry->replaceHttpReply(reply);
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * Parse a gopher request into components.  By Anawat.
  */
 static void
 gopher_request_parse(const HttpRequest * req, char *type_id, char *request)
 {
-    const char *path = req->urlpath.termedBuf();
+    ::Parser::Tokenizer tok(req->url.path());
 
     if (request)
-        request[0] = '\0';
+        *request = 0;
 
-    if (path && (*path == '/'))
-        ++path;
+    tok.skip('/'); // ignore failures? path could be ab-empty
 
-    if (!path || !*path) {
+    if (tok.atEnd()) {
         *type_id = GOPHER_DIRECTORY;
         return;
     }
 
-    *type_id = path[0];
+    static const CharacterSet anyByte("UTF-8",0x00, 0xFF);
+
+    SBuf typeId;
+    (void)tok.prefix(typeId, anyByte, 1); // never fails since !atEnd()
+    *type_id = typeId[0];
 
     if (request) {
-        xstrncpy(request, path + 1, MAX_URL);
+        SBufToCstring(request, tok.remaining().substr(0, MAX_URL-1));
         /* convert %xx to char */
         rfc1738_unescape(request);
     }
 }
 
 /**
- \ingroup ServerProtocolGopherAPI
  * Parse the request to determine whether it is cachable.
  *
- \param req Request data.
- \retval 0  Not cachable.
- \retval 1  Cachable.
+ * \param req   Request data.
+ * \retval 0    Not cachable.
+ * \retval 1    Cachable.
  */
 int
 gopherCachable(const HttpRequest * req)
@@ -310,7 +318,6 @@ gopherCachable(const HttpRequest * req)
     return cachable;
 }
 
-/// \ingroup ServerProtocolGopherInternal
 static void
 gopherHTMLHeader(StoreEntry * e, const char *title, const char *substring)
 {
@@ -324,7 +331,6 @@ gopherHTMLHeader(StoreEntry * e, const char *title, const char *substring)
     storeAppendPrintf(e, "</H1>\n");
 }
 
-/// \ingroup ServerProtocolGopherInternal
 static void
 gopherHTMLFooter(StoreEntry * e)
 {
@@ -337,7 +343,6 @@ gopherHTMLFooter(StoreEntry * e)
     storeAppendPrintf(e, "</ADDRESS></BODY></HTML>\n");
 }
 
-/// \ingroup ServerProtocolGopherInternal
 static void
 gopherEndHTML(GopherStateData * gopherState)
 {
@@ -354,9 +359,8 @@ gopherEndHTML(GopherStateData * gopherState)
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * Convert Gopher to HTML.
- \par
+ *
  * Borrow part of code from libwww2 came with Mosaic distribution.
  */
 static void
@@ -381,7 +385,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
 
     entry = gopherState->entry;
 
-    if (gopherState->conversion == gopher_ds::HTML_INDEX_PAGE) {
+    if (gopherState->conversion == GopherStateData::HTML_INDEX_PAGE) {
         char *html_url = html_quote(entry->url());
         gopherHTMLHeader(entry, "Gopher Index %s", html_url);
         storeAppendPrintf(entry,
@@ -396,7 +400,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
         return;
     }
 
-    if (gopherState->conversion == gopher_ds::HTML_CSO_PAGE) {
+    if (gopherState->conversion == GopherStateData::HTML_CSO_PAGE) {
         char *html_url = html_quote(entry->url());
         gopherHTMLHeader(entry, "CSO Search of %s", html_url);
         storeAppendPrintf(entry,
@@ -414,7 +418,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
     String outbuf;
 
     if (!gopherState->HTML_header_added) {
-        if (gopherState->conversion == gopher_ds::HTML_CSO_RESULT)
+        if (gopherState->conversion == GopherStateData::HTML_CSO_RESULT)
             gopherHTMLHeader(entry, "CSO Search Result", NULL);
         else
             gopherHTMLHeader(entry, "Gopher Menu", NULL);
@@ -471,9 +475,9 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
 
         switch (gopherState->conversion) {
 
-        case gopher_ds::HTML_INDEX_RESULT:
+        case GopherStateData::HTML_INDEX_RESULT:
 
-        case gopher_ds::HTML_DIR: {
+        case GopherStateData::HTML_DIR: {
             tline = line;
             gtype = *tline;
             ++tline;
@@ -618,7 +622,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
             break;
             }           /* HTML_DIR, HTML_INDEX_RESULT */
 
-        case gopher_ds::HTML_CSO_RESULT: {
+        case GopherStateData::HTML_CSO_RESULT: {
             if (line[0] == '-') {
                 int code, recno;
                 char *s_code, *s_recno, *result;
@@ -699,7 +703,6 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
     return;
 }
 
-/// \ingroup ServerProtocolGopherInternal
 static void
 gopherTimeout(const CommTimeoutCbParams &io)
 {
@@ -713,7 +716,6 @@ gopherTimeout(const CommTimeoutCbParams &io)
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * This will be called when data is ready to be read from fd.
  * Read until error or connection closed.
  */
@@ -753,8 +755,8 @@ gopherReadReply(const Comm::ConnectionPointer &conn, char *buf, size_t len, Comm
         delayId.bytesIn(len);
 #endif
 
-        kb_incr(&(statCounter.server.all.kbytes_in), len);
-        kb_incr(&(statCounter.server.other.kbytes_in), len);
+        statCounter.server.all.kbytes_in += len;
+        statCounter.server.other.kbytes_in += len;
     }
 
     debugs(10, 5, HERE << conn << " read len=" << len);
@@ -796,7 +798,7 @@ gopherReadReply(const Comm::ConnectionPointer &conn, char *buf, size_t len, Comm
         /* Connection closed; retrieval done. */
         /* flush the rest of data in temp buf if there is one. */
 
-        if (gopherState->conversion != gopher_ds::NORMAL)
+        if (gopherState->conversion != GopherStateData::NORMAL)
             gopherEndHTML(gopherState);
 
         entry->timestampsSet();
@@ -804,7 +806,7 @@ gopherReadReply(const Comm::ConnectionPointer &conn, char *buf, size_t len, Comm
         gopherState->fwd->complete();
         gopherState->serverConn->close();
     } else {
-        if (gopherState->conversion != gopher_ds::NORMAL) {
+        if (gopherState->conversion != GopherStateData::NORMAL) {
             gopherToHTML(gopherState, buf, len);
         } else {
             entry->append(buf, len);
@@ -816,7 +818,6 @@ gopherReadReply(const Comm::ConnectionPointer &conn, char *buf, size_t len, Comm
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * This will be called when request write is complete. Schedule read of reply.
  */
 static void
@@ -828,15 +829,15 @@ gopherSendComplete(const Comm::ConnectionPointer &conn, char *buf, size_t size,
 
     if (size > 0) {
         fd_bytes(conn->fd, size, FD_WRITE);
-        kb_incr(&(statCounter.server.all.kbytes_out), size);
-        kb_incr(&(statCounter.server.other.kbytes_out), size);
+        statCounter.server.all.kbytes_out += size;
+        statCounter.server.other.kbytes_out += size;
     }
 
     if (errflag) {
         ErrorState *err;
         err = new ErrorState(ERR_WRITE_ERROR, Http::scServiceUnavailable, gopherState->fwd->request);
         err->xerrno = xerrno;
-        err->port = gopherState->fwd->request->port;
+        err->port = gopherState->fwd->request->url.port();
         err->url = xstrdup(entry->url());
         gopherState->fwd->fail(err);
         gopherState->serverConn->close();
@@ -859,25 +860,25 @@ gopherSendComplete(const Comm::ConnectionPointer &conn, char *buf, size_t size,
 
     case GOPHER_DIRECTORY:
         /* we got to convert it first */
-        gopherState->conversion = gopher_ds::HTML_DIR;
+        gopherState->conversion = GopherStateData::HTML_DIR;
         gopherState->HTML_header_added = 0;
         break;
 
     case GOPHER_INDEX:
         /* we got to convert it first */
-        gopherState->conversion = gopher_ds::HTML_INDEX_RESULT;
+        gopherState->conversion = GopherStateData::HTML_INDEX_RESULT;
         gopherState->HTML_header_added = 0;
         break;
 
     case GOPHER_CSO:
         /* we got to convert it first */
-        gopherState->conversion = gopher_ds::HTML_CSO_RESULT;
+        gopherState->conversion = GopherStateData::HTML_CSO_RESULT;
         gopherState->cso_recno = 0;
         gopherState->HTML_header_added = 0;
         break;
 
     default:
-        gopherState->conversion = gopher_ds::NORMAL;
+        gopherState->conversion = GopherStateData::NORMAL;
         entry->flush();
     }
 
@@ -891,11 +892,10 @@ gopherSendComplete(const Comm::ConnectionPointer &conn, char *buf, size_t size,
 }
 
 /**
- \ingroup ServerProtocolGopherInternal
  * This will be called when connect completes. Write request.
  */
 static void
-gopherSendRequest(int fd, void *data)
+gopherSendRequest(int, void *data)
 {
     GopherStateData *gopherState = (GopherStateData *)data;
     char *buf = (char *)memAllocate(MEM_4K_BUF);
@@ -928,25 +928,12 @@ gopherSendRequest(int fd, void *data)
     gopherState->entry->makePublic();
 }
 
-/// \ingroup ServerProtocolGopherInternal
-CBDATA_TYPE(GopherStateData);
-
-/// \ingroup ServerProtocolGopherAPI
 void
 gopherStart(FwdState * fwd)
 {
-    StoreEntry *entry = fwd->entry;
-    GopherStateData *gopherState;
-    CBDATA_INIT_TYPE(GopherStateData);
-    gopherState = cbdataAlloc(GopherStateData);
-    gopherState->buf = (char *)memAllocate(MEM_4K_BUF);
+    GopherStateData *gopherState = new GopherStateData(fwd);
 
-    entry->lock("gopherState");
-    gopherState->entry = entry;
-
-    gopherState->fwd = fwd;
-
-    debugs(10, 3, "gopherStart: " << entry->url()  );
+    debugs(10, 3, gopherState->entry->url());
 
     ++ statCounter.server.all.requests;
 
@@ -965,12 +952,12 @@ gopherStart(FwdState * fwd)
         gopherMimeCreate(gopherState);
 
         if (gopherState->type_id == GOPHER_INDEX) {
-            gopherState->conversion = gopher_ds::HTML_INDEX_PAGE;
+            gopherState->conversion = GopherStateData::HTML_INDEX_PAGE;
         } else {
             if (gopherState->type_id == GOPHER_CSO) {
-                gopherState->conversion = gopher_ds::HTML_CSO_PAGE;
+                gopherState->conversion = GopherStateData::HTML_CSO_PAGE;
             } else {
-                gopherState->conversion = gopher_ds::HTML_INDEX_PAGE;
+                gopherState->conversion = GopherStateData::HTML_INDEX_PAGE;
             }
         }
 
