@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -13,11 +13,13 @@
 
 #include "base/RefCount.h"
 
+/// Cache Manager API
 namespace Mgr
 {
 
 class Action;
 class ActionCreator;
+class ActionPasswordList;
 class ActionProfile;
 class ActionWriter;
 class Command;
@@ -35,5 +37,11 @@ typedef ActionPointer (ClassActionCreationHandler)(const CommandPointer &cmd);
 
 } // namespace Mgr
 
+class StoreEntry;
+/**
+ * Handler for "dumping" out a cachemgr report to a StoreEntry
+ */
+typedef void OBJH(StoreEntry *);
+
 #endif /* SQUID_MGR_FORWARD_H */
 
