@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -10,6 +10,8 @@
 #define SQUID_PCONN_H
 
 #include "base/CbcPointer.h"
+#include "mgr/forward.h"
+
 #include <set>
 
 /**
@@ -22,9 +24,7 @@
 class PconnPool;
 class PeerPoolMgr;
 
-/* for CBDATA_CLASS2() macros */
 #include "cbdata.h"
-/* for hash_link */
 #include "hash.h"
 /* for IOCB */
 #include "comm.h"
@@ -37,6 +37,8 @@ class PeerPoolMgr;
  */
 class IdleConnList
 {
+    CBDATA_CLASS(IdleConnList);
+
 public:
     IdleConnList(const char *key, PconnPool *parent);
     ~IdleConnList();
@@ -92,8 +94,6 @@ private:
     PconnPool *parent_;
 
     char fakeReadBuf_[4096]; // TODO: kill magic number.
-
-    CBDATA_CLASS2(IdleConnList);
 };
 
 #include "ip/forward.h"
