@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -34,6 +34,7 @@
 #include "Store.h"
 #include "store_digest.h"
 #include "store_key_md5.h"
+#include "store_key_md5.h"
 #include "store_log.h"
 #include "store_rebuild.h"
 #include "StoreClient.h"
@@ -47,6 +48,11 @@
 #include "DelayPools.h"
 #endif
 
+/** StoreEntry uses explicit new/delete operators, which set pool chunk size to 2MB
+ * XXX: convert to MEMPROXY_CLASS() API
+ */
+#include "mem/Pool.h"
+
 #include <climits>
 #include <stack>
 
@@ -121,7 +127,7 @@ Store::Root(StorePointer aRoot)
 void
 Store::Stats(StoreEntry * output)
 {
-    assert (output);
+    assert(output);
     Root().stat(*output);
 }
 
@@ -138,36 +144,18 @@ Store::sync()
 {}
 
 void
-Store::unlink (StoreEntry &anEntry)
+Store::unlink(StoreEntry &)
 {
     fatal("Store::unlink on invalid Store\n");
 }
 
-void *
-StoreEntry::operator new (size_t bytecount)
-{
-    assert (bytecount == sizeof (StoreEntry));
-
-    if (!pool) {
-        pool = memPoolCreate ("StoreEntry", bytecount);
-    }
-
-    return pool->alloc();
-}
-
 void
-StoreEntry::operator delete (void *address)
-{
-    pool->freeOne(address);
-}
-
-void
-StoreEntry::makePublic(const KeyScope scope)
+StoreEntry::makePublic()
 {
     /* This object can be cached for a long time */
 
     if (!EBIT_TEST(flags, RELEASE_REQUEST))
-        setPublicKey(scope);
+        setPublicKey();
 }
 
 void
@@ -272,13 +260,13 @@ StoreEntry::bytesWanted (Range<size_t> const aRange, bool ignoreDelayPools) cons
 }
 
 bool
-StoreEntry::checkDeferRead(int fd) const
+StoreEntry::checkDeferRead(int) const
 {
     return (bytesWanted(Range<size_t>(0,INT_MAX)) == 0);
 }
 
 void
-StoreEntry::setNoDelay (bool const newValue)
+StoreEntry::setNoDelay(bool const newValue)
 {
     if (mem_obj)
         mem_obj->setNoDelay(newValue);
@@ -355,7 +343,7 @@ StoreEntry::StoreEntry() :
     timestamp(-1),
     lastref(-1),
     expires(-1),
-    lastModified_(-1),
+    lastmod(-1),
     swap_file_sz(0),
     refcount(0),
     flags(0),
@@ -585,19 +573,19 @@ storeGetPublic(const char *uri, const HttpRequestMethod& method)
 }
 
 StoreEntry *
-storeGetPublicByRequestMethod(HttpRequest * req, const HttpRequestMethod& method, const KeyScope keyScope)
+storeGetPublicByRequestMethod(HttpRequest * req, const HttpRequestMethod& method)
 {
-    return Store::Root().get(storeKeyPublicByRequestMethod(req, method, keyScope));
+    return Store::Root().get(storeKeyPublicByRequestMethod(req, method));
 }
 
 StoreEntry *
-storeGetPublicByRequest(HttpRequest * req, const KeyScope keyScope)
+storeGetPublicByRequest(HttpRequest * req)
 {
-    StoreEntry *e = storeGetPublicByRequestMethod(req, req->method, keyScope);
+    StoreEntry *e = storeGetPublicByRequestMethod(req, req->method);
 
     if (e == NULL && req->method == Http::METHOD_HEAD)
         /* We can generate a HEAD reply from a cached GET object */
-        e = storeGetPublicByRequestMethod(req, Http::METHOD_GET, keyScope);
+        e = storeGetPublicByRequestMethod(req, Http::METHOD_GET);
 
     return e;
 }
@@ -653,8 +641,10 @@ StoreEntry::setPrivateKey()
 }
 
 void
-StoreEntry::setPublicKey(const KeyScope scope)
+StoreEntry::setPublicKey()
 {
+    const cache_key *newkey;
+
     if (key && !EBIT_TEST(flags, KEY_PRIVATE))
         return;                 /* is already public */
 
@@ -678,35 +668,84 @@ StoreEntry::setPublicKey(const KeyScope scope)
 
     assert(!EBIT_TEST(flags, RELEASE_REQUEST));
 
-    adjustVary();
-    forcePublicKey(calcPublicKey(scope));
-}
+    if (mem_obj->request) {
+        HttpRequest *request = mem_obj->request;
 
-void
-StoreEntry::clearPublicKeyScope()
-{
-    if (!key || EBIT_TEST(flags, KEY_PRIVATE))
-        return; // probably the old public key was deleted or made private
+        if (!mem_obj->vary_headers) {
+            /* First handle the case where the object no longer varies */
+            safe_free(request->vary_headers);
+        } else {
+            if (request->vary_headers && strcmp(request->vary_headers, mem_obj->vary_headers) != 0) {
+                /* Oops.. the variance has changed. Kill the base object
+                 * to record the new variance key
+                 */
+                safe_free(request->vary_headers);       /* free old "bad" variance key */
+                if (StoreEntry *pe = storeGetPublic(mem_obj->storeId(), mem_obj->method))
+                    pe->release();
+            }
 
-    // TODO: adjustVary() when collapsed revalidation supports that
+            /* Make sure the request knows the variance status */
+            if (!request->vary_headers) {
+                const char *vary = httpMakeVaryMark(request, mem_obj->getReply());
 
-    const cache_key *newKey = calcPublicKey(ksDefault);
-    if (!storeKeyHashCmp(key, newKey))
-        return; // probably another collapsed revalidation beat us to this change
+                if (vary)
+                    request->vary_headers = xstrdup(vary);
+            }
+        }
 
-    forcePublicKey(newKey);
-}
+        // TODO: storeGetPublic() calls below may create unlocked entries.
+        // We should add/use storeHas() API or lock/unlock those entries.
+        if (mem_obj->vary_headers && !storeGetPublic(mem_obj->storeId(), mem_obj->method)) {
+            /* Create "vary" base object */
+            String vary;
+            StoreEntry *pe = storeCreateEntry(mem_obj->storeId(), mem_obj->logUri(), request->flags, request->method);
+            /* We are allowed to do this typecast */
+            HttpReply *rep = new HttpReply;
+            rep->setHeaders(Http::scOkay, "Internal marker object", "x-squid-internal/vary", -1, -1, squid_curtime + 100000);
+            vary = mem_obj->getReply()->header.getList(Http::HdrType::VARY);
+
+            if (vary.size()) {
+                /* Again, we own this structure layout */
+                rep->header.putStr(Http::HdrType::VARY, vary.termedBuf());
+                vary.clean();
+            }
+
+#if X_ACCELERATOR_VARY
+            vary = mem_obj->getReply()->header.getList(Http::HdrType::HDR_X_ACCELERATOR_VARY);
+
+            if (vary.size() > 0) {
+                /* Again, we own this structure layout */
+                rep->header.putStr(Http::HdrType::HDR_X_ACCELERATOR_VARY, vary.termedBuf());
+                vary.clean();
+            }
+
+#endif
+            pe->replaceHttpReply(rep, false); // no write until key is public
+
+            pe->timestampsSet();
+
+            pe->makePublic();
+
+            pe->startWriting(); // after makePublic()
+
+            pe->complete();
+
+            pe->unlock("StoreEntry::setPublicKey+Vary");
+        }
+
+        newkey = storeKeyPublicByRequest(mem_obj->request);
+    } else
+        newkey = storeKeyPublic(mem_obj->storeId(), mem_obj->method);
 
-/// Unconditionally sets public key for this store entry.
-/// Releases the old entry with the same public key (if any).
-void
-StoreEntry::forcePublicKey(const cache_key *newkey)
-{
     if (StoreEntry *e2 = (StoreEntry *)hash_lookup(store_table, newkey)) {
-        assert(e2 != this);
         debugs(20, 3, "Making old " << *e2 << " private.");
         e2->setPrivateKey();
         e2->release();
+
+        if (mem_obj->request)
+            newkey = storeKeyPublicByRequest(mem_obj->request);
+        else
+            newkey = storeKeyPublic(mem_obj->storeId(), mem_obj->method);
     }
 
     if (key)
@@ -720,84 +759,6 @@ StoreEntry::forcePublicKey(const cache_key *newkey)
         storeDirSwapLog(this, SWAP_LOG_ADD);
 }
 
-/// Calculates correct public key for feeding forcePublicKey().
-/// Assumes adjustVary() has been called for this entry already.
-const cache_key *StoreEntry::calcPublicKey(const KeyScope keyScope) {
-    assert(mem_obj);
-    return mem_obj->request ?  storeKeyPublicByRequest(mem_obj->request, keyScope) :
-           storeKeyPublic(mem_obj->storeId(), mem_obj->method, keyScope);
-}
-
-/// Updates mem_obj->request->vary_headers to reflect the current Vary.
-/// The vary_headers field is used to calculate the Vary marker key.
-/// Releases the old Vary marker with an outdated key (if any).
-void StoreEntry::adjustVary() {
-    assert(mem_obj);
-
-    if (!mem_obj->request)
-        return;
-
-    HttpRequest *request = mem_obj->request;
-
-    if (mem_obj->vary_headers.isEmpty()) {
-        /* First handle the case where the object no longer varies */
-        request->vary_headers.clear();
-    } else {
-        if (!request->vary_headers.isEmpty() && request->vary_headers.cmp(mem_obj->vary_headers) != 0) {
-            /* Oops.. the variance has changed. Kill the base object
-             * to record the new variance key
-             */
-            request->vary_headers.clear();       /* free old "bad" variance key */
-            if (StoreEntry *pe = storeGetPublic(mem_obj->storeId(), mem_obj->method))
-                pe->release();
-        }
-
-        /* Make sure the request knows the variance status */
-        if (request->vary_headers.isEmpty())
-            request->vary_headers = httpMakeVaryMark(request, mem_obj->getReply());
-    }
-
-    // TODO: storeGetPublic() calls below may create unlocked entries.
-    // We should add/use storeHas() API or lock/unlock those entries.
-    if (!mem_obj->vary_headers.isEmpty() && !storeGetPublic(mem_obj->storeId(), mem_obj->method)) {
-        /* Create "vary" base object */
-        String vary;
-        StoreEntry *pe = storeCreateEntry(mem_obj->storeId(), mem_obj->logUri(), request->flags, request->method);
-        /* We are allowed to do this typecast */
-        HttpReply *rep = new HttpReply;
-        rep->setHeaders(Http::scOkay, "Internal marker object", "x-squid-internal/vary", -1, -1, squid_curtime + 100000);
-        vary = mem_obj->getReply()->header.getList(HDR_VARY);
-
-        if (vary.size()) {
-            /* Again, we own this structure layout */
-            rep->header.putStr(HDR_VARY, vary.termedBuf());
-            vary.clean();
-        }
-
-#if X_ACCELERATOR_VARY
-        vary = mem_obj->getReply()->header.getList(HDR_X_ACCELERATOR_VARY);
-
-        if (vary.size() > 0) {
-            /* Again, we own this structure layout */
-            rep->header.putStr(HDR_X_ACCELERATOR_VARY, vary.termedBuf());
-            vary.clean();
-        }
-
-#endif
-        pe->replaceHttpReply(rep, false); // no write until key is public
-
-        pe->timestampsSet();
-
-        pe->makePublic();
-
-        pe->startWriting(); // after makePublic()
-
-        pe->complete();
-
-        pe->unlock("StoreEntry::forcePublicKey+Vary");
-    }
-}
-
 StoreEntry *
 storeCreatePureEntry(const char *url, const char *log_url, const RequestFlags &flags, const HttpRequestMethod& method)
 {
@@ -885,24 +846,62 @@ StoreEntry::append(char const *buf, int len)
     write(tempBuffer);
 }
 
+void
+StoreEntry::vappendf(const char *fmt, va_list vargs)
+{
+    LOCAL_ARRAY(char, buf, 4096);
+    *buf = 0;
+    int x;
+
+#ifdef VA_COPY
+    va_args ap;
+    /* Fix of bug 753r. The value of vargs is undefined
+     * after vsnprintf() returns. Make a copy of vargs
+     * incase we loop around and call vsnprintf() again.
+     */
+    VA_COPY(ap,vargs);
+    errno = 0;
+    if ((x = vsnprintf(buf, sizeof(buf), fmt, ap)) < 0) {
+        fatal(xstrerr(errno));
+        return;
+    }
+    va_end(ap);
+#else /* VA_COPY */
+    errno = 0;
+    if ((x = vsnprintf(buf, sizeof(buf), fmt, vargs)) < 0) {
+        fatal(xstrerr(errno));
+        return;
+    }
+#endif /*VA_COPY*/
+
+    if (x < static_cast<int>(sizeof(buf))) {
+        append(buf, x);
+        return;
+    }
+
+    // okay, do it the slow way.
+    char *buf2 = new char[x+1];
+    int y = vsnprintf(buf2, x+1, fmt, vargs);
+    assert(y >= 0 && y == x);
+    append(buf2, y);
+    delete[] buf2;
+}
+
+// deprecated. use StoreEntry::appendf() instead.
 void
 storeAppendPrintf(StoreEntry * e, const char *fmt,...)
 {
     va_list args;
     va_start(args, fmt);
-
-    storeAppendVPrintf(e, fmt, args);
+    e->vappendf(fmt, args);
     va_end(args);
 }
 
-/* used be storeAppendPrintf and Packer */
+// deprecated. use StoreEntry::appendf() instead.
 void
 storeAppendVPrintf(StoreEntry * e, const char *fmt, va_list vargs)
 {
-    LOCAL_ARRAY(char, buf, 4096);
-    buf[0] = '\0';
-    vsnprintf(buf, 4096, fmt, vargs);
-    e->append(buf, strlen(buf));
+    e->vappendf(fmt, vargs);
 }
 
 struct _store_check_cachable_hist {
@@ -1221,7 +1220,7 @@ storeGetMemSpace(int size)
  * it becomes active will self register
  */
 void
-Store::Maintain(void *notused)
+Store::Maintain(void *)
 {
     Store::Root().maintain();
 
@@ -1313,7 +1312,7 @@ StoreEntry::release()
 }
 
 static void
-storeLateRelease(void *unused)
+storeLateRelease(void *)
 {
     StoreEntry *e;
     static int n = 0;
@@ -1580,12 +1579,12 @@ StoreEntry::validToSend() const
     return 1;
 }
 
-bool
+void
 StoreEntry::timestampsSet()
 {
     const HttpReply *reply = getReply();
     time_t served_date = reply->date;
-    int age = reply->header.getInt(HDR_AGE);
+    int age = reply->header.getInt(Http::HdrType::AGE);
     /* Compute the timestamp, mimicking RFC2616 section 13.2.3. */
     /* make sure that 0 <= served_date <= squid_curtime */
 
@@ -1618,27 +1617,14 @@ StoreEntry::timestampsSet()
             served_date -= (squid_curtime - request_sent);
     }
 
-    time_t exp = 0;
     if (reply->expires > 0 && reply->date > -1)
-        exp = served_date + (reply->expires - reply->date);
+        expires = served_date + (reply->expires - reply->date);
     else
-        exp = reply->expires;
+        expires = reply->expires;
 
-    if (timestamp == served_date && expires == exp) {
-        // if the reply lacks LMT, then we now know that our effective
-        // LMT (i.e., timestamp) will stay the same, otherwise, old and
-        // new modification times must match
-        if (reply->last_modified < 0 || reply->last_modified == lastModified())
-            return false; // nothing has changed
-    }
-
-    expires = exp;
-
-    lastModified_ = reply->last_modified;
+    lastmod = reply->last_modified;
 
     timestamp = served_date;
-
-    return true;
 }
 
 void
@@ -1669,7 +1655,7 @@ StoreEntry::dump(int l) const
     debugs(20, l, "StoreEntry->timestamp: " << timestamp);
     debugs(20, l, "StoreEntry->lastref: " << lastref);
     debugs(20, l, "StoreEntry->expires: " << expires);
-    debugs(20, l, "StoreEntry->lastModified_: " << lastModified_);
+    debugs(20, l, "StoreEntry->lastmod: " << lastmod);
     debugs(20, l, "StoreEntry->swap_file_sz: " << swap_file_sz);
     debugs(20, l, "StoreEntry->refcount: " << refcount);
     debugs(20, l, "StoreEntry->flags: " << storeEntryFlags(this));
@@ -1751,14 +1737,21 @@ StoreEntry::createMemObject(const char *aUrl, const char *aLogUrl, const HttpReq
     mem_obj->setUris(aUrl, aLogUrl, aMethod);
 }
 
-/* this just sets DELAY_SENDING */
+/** disable sending content to the clients.
+ *
+ * This just sets DELAY_SENDING.
+ */
 void
 StoreEntry::buffer()
 {
     EBIT_SET(flags, DELAY_SENDING);
 }
 
-/* this just clears DELAY_SENDING and Invokes the handlers */
+/** flush any buffered content.
+ *
+ * This just clears DELAY_SENDING and Invokes the handlers
+ * to begin sending anything that may be buffered.
+ */
 void
 StoreEntry::flush()
 {
@@ -1800,7 +1793,7 @@ StoreEntry::reset()
     mem_obj->reset();
     HttpReply *rep = (HttpReply *) getReply();       // bypass const
     rep->reset();
-    expires = lastModified_ = timestamp = -1;
+    expires = lastmod = timestamp = -1;
 }
 
 /*
@@ -1919,12 +1912,9 @@ StoreEntry::replaceHttpReply(HttpReply *rep, bool andStartWriting)
 void
 StoreEntry::startWriting()
 {
-    Packer p;
-
-    /* TODO: when we store headers serparately remove the header portion */
+    /* TODO: when we store headers separately remove the header portion */
     /* TODO: mark the length of the headers ? */
     /* We ONLY want the headers */
-    packerToStoreInit(&p, this);
 
     assert (isEmpty());
     assert(mem_obj);
@@ -1932,13 +1922,13 @@ StoreEntry::startWriting()
     const HttpReply *rep = getReply();
     assert(rep);
 
-    rep->packHeadersInto(&p);
+    buffer();
+    rep->packHeadersInto(this);
     mem_obj->markEndOfReplyHeaders();
     EBIT_CLR(flags, ENTRY_FWD_HDR_WAIT);
 
-    rep->body.packInto(&p);
-
-    packerClean(&p);
+    rep->body.packInto(this);
+    flush();
 }
 
 char const *
@@ -1973,7 +1963,7 @@ StoreEntry::transientsAbandonmentCheck()
 }
 
 void
-StoreEntry::memOutDecision(const bool willCacheInRam)
+StoreEntry::memOutDecision(const bool)
 {
     transientsAbandonmentCheck();
 }
@@ -2010,10 +2000,13 @@ StoreEntry::trimMemory(const bool preserveSwappable)
 }
 
 bool
-StoreEntry::modifiedSince(const time_t ims, const int imslen) const
+StoreEntry::modifiedSince(HttpRequest * request) const
 {
     int object_length;
-    const time_t mod_time = lastModified();
+    time_t mod_time = lastmod;
+
+    if (mod_time < 0)
+        mod_time = timestamp;
 
     debugs(88, 3, "modifiedSince: '" << url() << "'");
 
@@ -2028,16 +2021,16 @@ StoreEntry::modifiedSince(const time_t ims, const int imslen) const
     if (object_length < 0)
         object_length = contentLen();
 
-    if (mod_time > ims) {
+    if (mod_time > request->ims) {
         debugs(88, 3, "--> YES: entry newer than client");
         return true;
-    } else if (mod_time < ims) {
+    } else if (mod_time < request->ims) {
         debugs(88, 3, "-->  NO: entry older than client");
         return false;
-    } else if (imslen < 0) {
+    } else if (request->imslen < 0) {
         debugs(88, 3, "-->  NO: same LMT, no client length");
         return false;
-    } else if (imslen == object_length) {
+    } else if (request->imslen == object_length) {
         debugs(88, 3, "-->  NO: same LMT, same length");
         return false;
     } else {
@@ -2050,7 +2043,7 @@ bool
 StoreEntry::hasEtag(ETag &etag) const
 {
     if (const HttpReply *reply = getReply()) {
-        etag = reply->header.getETag(HDR_ETAG);
+        etag = reply->header.getETag(Http::HdrType::ETAG);
         if (etag.str)
             return true;
     }
@@ -2060,14 +2053,14 @@ StoreEntry::hasEtag(ETag &etag) const
 bool
 StoreEntry::hasIfMatchEtag(const HttpRequest &request) const
 {
-    const String reqETags = request.header.getList(HDR_IF_MATCH);
+    const String reqETags = request.header.getList(Http::HdrType::IF_MATCH);
     return hasOneOfEtags(reqETags, false);
 }
 
 bool
 StoreEntry::hasIfNoneMatchEtag(const HttpRequest &request) const
 {
-    const String reqETags = request.header.getList(HDR_IF_NONE_MATCH);
+    const String reqETags = request.header.getList(Http::HdrType::IF_NONE_MATCH);
     // weak comparison is allowed only for HEAD or full-body GET requests
     const bool allowWeakMatch = !request.flags.isRanged &&
                                 (request.method == Http::METHOD_GET || request.method == Http::METHOD_HEAD);
@@ -2078,7 +2071,7 @@ StoreEntry::hasIfNoneMatchEtag(const HttpRequest &request) const
 bool
 StoreEntry::hasOneOfEtags(const String &reqETags, const bool allowWeakMatch) const
 {
-    const ETag repETag = getReply()->header.getETag(HDR_ETAG);
+    const ETag repETag = getReply()->header.getETag(Http::HdrType::ETAG);
     if (!repETag.str)
         return strListIsMember(&reqETags, "*", ',');
 
@@ -2134,18 +2127,6 @@ StoreEntry::isAccepting() const
     return true;
 }
 
-const char *
-StoreEntry::describeTimestamps() const
-{
-    LOCAL_ARRAY(char, buf, 256);
-    snprintf(buf, 256, "LV:%-9d LU:%-9d LM:%-9d EX:%-9d",
-             static_cast<int>(timestamp),
-             static_cast<int>(lastref),
-             static_cast<int>(lastModified_),
-             static_cast<int>(expires));
-    return buf;
-}
-
 std::ostream &operator <<(std::ostream &os, const StoreEntry &e)
 {
     os << "e:";
@@ -2175,11 +2156,10 @@ std::ostream &operator <<(std::ostream &os, const StoreEntry &e)
     // print only set flags, using unique letters
     if (e.flags) {
         if (EBIT_TEST(e.flags, ENTRY_SPECIAL)) os << 'S';
-        if (EBIT_TEST(e.flags, ENTRY_REVALIDATE_ALWAYS)) os << 'R';
+        if (EBIT_TEST(e.flags, ENTRY_REVALIDATE)) os << 'R';
         if (EBIT_TEST(e.flags, DELAY_SENDING)) os << 'P';
         if (EBIT_TEST(e.flags, RELEASE_REQUEST)) os << 'X';
         if (EBIT_TEST(e.flags, REFRESH_REQUEST)) os << 'F';
-        if (EBIT_TEST(e.flags, ENTRY_REVALIDATE_STALE)) os << 'E';
         if (EBIT_TEST(e.flags, ENTRY_DISPATCHED)) os << 'D';
         if (EBIT_TEST(e.flags, KEY_PRIVATE)) os << 'I';
         if (EBIT_TEST(e.flags, ENTRY_FWD_HDR_WAIT)) os << 'W';
