@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -10,8 +10,9 @@
 
 #include <cppunit/TestAssert.h>
 
-#include "Mem.h"
+#include "Debug.h"
 #include "testURL.h"
+#include "unitTestMain.h"
 #include "URL.h"
 
 #include <sstream>
