@@ -1,55 +1,28 @@
 /*
- * DEBUG: section 00    UFS Store Dump Tool
- * AUTHOR: Robert Collins
- *
- * SQUID Web Proxy Cache          http://www.squid-cache.org/
- * ----------------------------------------------------------
- *
- *  Squid is the result of efforts by numerous individuals from
- *  the Internet community; see the CONTRIBUTORS file for full
- *  details.   Many organizations have provided support for Squid's
- *  development; see the SPONSORS file for full details.  Squid is
- *  Copyrighted (C) 2001 by the Regents of the University of
- *  California; see the COPYRIGHT file for full details.  Squid
- *  incorporates software developed and/or copyrighted by other
- *  sources; see the CREDITS file for full details.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
+/* DEBUG: section 00    UFS Store Dump Tool */
+
 #include "squid.h"
-#include "StoreMeta.h"
-#include "StoreMetaUnpacker.h"
+#include "Generic.h"
+#include "md5.h"
+#include "mgr/Registration.h"
 #include "Store.h"
 #include "store_key_md5.h"
-#include "Generic.h"
-#include "mgr/Registration.h"
+#include "StoreMeta.h"
+#include "StoreMetaUnpacker.h"
 
 #undef malloc
 #undef free
 
-#if HAVE_STDEXCEPT
-#include <stdexcept>
-#endif
-#if HAVE_IOSTREAM
-#include <iostream>
-#endif
-#if HAVE_CASSERT
 #include <cassert>
-#endif
+#include <iostream>
+#include <stdexcept>
 
 /* stub functions for parts of squid not factored to be dynamic yet */
 void
@@ -201,3 +174,4 @@ main(int argc, char *argv[])
         return 1;
     }
 }
+
